/* This software provides the support for sets.  A set has three
   elements:

   1) cardinality
   2) a set name, and
   3) a pointer to the first element

   Three additional elements are added to the defination of a set for
   the purpose of determining the load balancing:

   4) number of operations to factorize this independent block
   5) number of operations to factorize this portion of the border
   6) indicator variable to mark whether cluster has been used in
      a load-balanced super cluster

   Memory is allocated dynamically.  For set elements, memory is first
   used that has been returned to a memory pool, and only if no
   memory is available, the memory is obtained using calloc().
   No support, however, is provided for saving memory that is allocated
   for sets, sets of sets, and sets of sets elements. */

#ifndef SETS_H
#define SETS_H

/* define a data type for a set element */
typedef struct SetElement
{
   int             Element;
   struct SetElement *Next;
}               SetElement;

/* define a data type for a set */
typedef struct Set
{
   int             n;
   char           *Name;
   int             nOpsBlockGS;
   int             nOpsBorderGS;
   int             nOpsTotalGS;
   int             nOpsTotal;
   int             SuperCluster;
   SetElement     *First;
}               Set;

/* define a data type for a set-of-sets-element */
typedef struct SoSElement
{
   Set            *set;
   struct SoSElement *Next;
}               SoSElement;

/* The clusters will be stored in a set of sets */
typedef struct SetofSets
{
   int             n;
   char           *Name;
   SoSElement     *First
}               SoSets;

/* define a set called SetElementPool that will be used to collect
   and recycle memory for discarded set elements */

#ifdef SETS_C
Set            *SetElementPool;
SoSets         *SetOfSetsElementPool;
#else
extern Set            *SetElementPool;
extern SoSets         *SetOfSetsElementPool;
#endif

#endif

SetElement     *SetElementCalloc();
void            SetElementFree();
SoSElement     *SetOfSetsElementCalloc();
void            SetOfSetsElementFree();

Set            *InitSet();
void            DeleteSet();
SoSets         *InitSetOfSets();
void            UnionOfSets();
void            DifferenceOfSets();
void            AddElementToSet();
void            DeleteElementFromSet();
int             MinValueInSet();
int             ElementOfSet();
void            PrintSet();
void            AddSetToSetOfSets();
void            PrintSetOfSets();

